from setuptools import setup

setup(
    name='lunar_python',
    version='1.2.7',
    packages=['lunar_python', 'lunar_python.util', 'lunar_python.eightchar'],
    url='https://github.com/6tail/lunar-python',
    license='MIT',
    author='6tail',
    author_email='6tail@6tail.cn',
    description='lunar is a calendar library for Solar and Chinese Lunar.',
    long_description='lunar is a calendar library for Solar and Chinese Lunar.',
    keywords='solar lunar'
)
